(* :Title: Nine *)

(* :Context: SpinAlgebra`NineJ` *)

(* :Author: Wojciech Broniowski *)

(* :Summary:
   Computes 6j and 9j coefficients *)


(* :Keywords: 6j, 9j, Wigner, spin algebra, angular momentum, recoupling coefficients *)


BeginPackage["SpinAlgebra`NineJ`"]

J6::usage = "J6[{j1,j2,j12},{j3,J,j23}] is the Wigner 6j coefficient"
J9::usage = "J9[{j1,j2,j12},{j3,j4,j34},{j13,j24,J}] is the Wigner 6j coefficient"


Begin["`Private`"]


(* Print["Calculates the Wigner 6j and 9j coefficients"] *)
Print["Author: Wojciech Broniowski"]

Print["Usage:"]
Print["  J6[{j1,j2,j12},{j3,J,j23}]"]
Print["  J9[{j1,j2,j12},{j3,j4,j34},{j13,j24,J}]"]

Triangle[a_,b_,c_] := True  /;((a <= b+c) && (a >= Abs[b-c]))
Triangle[a_,b_,c_] := False /;!((a <= b+c) && (a >= Abs[b-c]))

J6[{j1_,j2_,j12_},{j3_,J_,j23_}] :=
SixJSymbol[{j1,j2,j12},{j3,J,j23}] /; (Triangle[j1,j2,j12] &&
Triangle[j2,j3,j23] && Triangle[j1,j23,J] && Triangle[j12,j3,J])

J6[{j1_,j2_,j12_},{j3_,J_,j23_}] :=
0 /; !(Triangle[j1,j2,j12] &&
Triangle[j2,j3,j23] && Triangle[j1,j23,J] && Triangle[j12,j3,J])

J9[{j1_,j2_,j12_},{j3_,j4_,j34_},{j13_,j24_,J_}] := Module[{}, 
jmin:=Min[Abs[j1-J],Abs[j24-j3],Abs[j34-j2]];
jmax:=Max[j1+J,j24+j3,j34+j2]; 
Expand[Sum[(-1)^(2 j)(2 j+1) J6[{j1,j3,j13},{j24,J,j}] J6[{j2,j4,j24},{j3,j,j34}] J6[{j12,j34,J},{j,j1,j2}], 
{j,jmin,jmax}]]] 

End[]

EndPackage[]