(*:Mathematica Version: 5.0 *)

(*:Package Version: 1.30 *)

(*:Name: Utilities`Plot` *)

(*:Author: Eric W. Weisstein
           addPointsAndMapToDisk by Michael Trott
           original PlotRoots by Rob Knapp
           original PerfectPixel by Oyvind Tafjord
           BasePlot based on an idea by Ed Pegg, Jr., 2003-07-14
           RealPlot by Adam Strzebonski, 2004-01-14
*)

(*:URL:
  http://library.wolfram.com/infocenter/MathSource/5087/
*)

(*:Summary:
*)

(*:History:
  v1.00 (1997      ): Written by Eric W. Weisstein
  v1.01 (1999-06-20): Documentation updated
  v1.02 (1999-11-18): Updated to use TraditionalForm
  v1.03 (2000-01-01): URL updated
  v1.04 (2000-04-15): Package renamed from PlotReIm` to Plot`.
                      PlotDiscontinuous.
  v1.05 (2000-07-08): RadialContourPlot
  v1.06 (2000-08-08): ComplexContourPlot
  v1.07 (2000-08-20): HoldForm added to ComplexContourPlot
  v1.08 (2000-08-27): Added title option to ComplexContourPlot
  v1.09 (2002-02-26): Cleaned up argumentsd and improved syntax to PlotDiscontinuous
  v1.10 (2002-11-04): Cleaned up, renamed, and standardized ComplexContourPlot and
                      ComplexPlot
  v1.11 (2002-11-08): Now allow nice specification of individual plot titles using
                      ComplexPlotLabel
  v1.12 (2002-11-11): IntegerListPlot
  v1.13 (2002-11-24): Rewrote PlotDiscontinuous; added default options
  v1.14 (2002-11-25): Fixed options to ComplexContourPlot
  v1.15 (2002-12-05): PlotRoot added (written by Rob Knapp, rknapp@wolfram.com)
  v1.16 (2002-12-08): Added Center option to IntegerListPlot
  v1.17 (2003-04-05): Modified RadialContourPlot to use functions with variables
                      instead of pure functions
  v1.18 (2003-05-04): Moved PixelPerfect and RasterGraphics here
  v1.19 (2003-05-31): TrigPlot
  v1.20 (2003-06-27): Slightly updated docs
  v1.21 (2003-08-05): AsymptoteStyle
  v1.22 (2003-08-14): PointStyle
  v1.23 (2003-09-08): Suppress debugging Print statement
  v1.24 (2003-10-27): BasePlot
  v1.25 (2003-12-18): BranchCutPlot
  v1.26 (2004-01-04): Renamed BasePlot to NaryPlot
  v1.27 (2004-01-14): RealPlot
  v1.28 (2004-05-28): Optimized ComplexPlot3D, ComplexContourPlot,
                      extended IntegerListPlot to non-zero offset
  v1.29 (2004-06-01): Interval boundary points added to PlotDiscontinuous
  v1.30 (2004-08-10): Corrected List* versions of ComplexPlot3D and ComplexContourPlot
                      to do option processing correctly
  
  (c) 1997-2004 Eric W. Weisstein
*)

(*:Keywords:
  
*)

(*:Requirements: None. *)

(*:Discussion:

Package to implement some additional types of 1- and 2-D plots that are useful 
for creating MathWorld figures.

*)

(*:References: *)

(*:Limitations: 

Usage messages need to be added for PixelPerfect options.

PlotDiscontinuous numericization is a bit dodgy, but this is probably very difficult
(or impossible) to treat correctly in all generality

*)

BeginPackage["Plot`",{
	"MathWorld`SpecialFunctions`",        (* Only needed for BranchCutPlot *)
	"LinearAlgebra`MatrixManipulation`",
	"Utilities`FilterOptions`",
	"Graphics`Colors`"
	}
]

AnglePosition::usage =
"AngelPosition is an option to TrigPlot specifying where to manually place \
the angle label."

AsymptoteStyle::usage =
"AsymptoteStyle is an option for PlotDiscontinuous."

PixelsPerCell::usage =
""

BlockSize::usage =
""

Blocking::usage =
""

BranchCutPlot::usage =
"BranchCutPlot[f] makes a plot of the branch cuts of the function \
f in the complex plane.  These are defined in MathWorld`SpecialFunctions`."

BranchStyle::usage =
"BranchStyle->style is an option to BranchCutPlot."

Frame::usage =
""

Margin::usage =
""

MaxSize::usage =
""

MaxPixelsPerCell::usage =
""

ClosedEndpoints::usage =
"ClosedEndpoints is an option to PlotDiscontinuous that takes a list of points \
to be plotted with closed dots."

ComplexContourPlot::usage =
"ComplexContourPlot[f,{z,zmin,zmax},<opts>] makes a GraphicsArray of contour plots \
of the real part, imaginary part, and absolute value of the complex function \
f for zmin < z < zmax, where zmin and zmax are specified as complex numbers.  \
If they are real numbers, then the range is taken to be the square \
(1+I)zmin < z < (1+I)zmax.  Options can be specified, including PlotPoints->pts."

ComplexPlot3D::usage =
"ComplexPlot3D[f,{z,zmin,zmax},<opts>] make a GraphicsArray of height plots \
of the real part, imaginary part, and absolute value of the complex function f
for zmin < z < zmax, where zmin and zmax are specified as comple numbers.  \
If they are real numbers, then the range is taken to be the square \
(1+I)zmin < z < (1+I)zmax.  Options can be specified, including PlotPoints->pts, \
Mesh->False, etc."

ComplexPlotLabel::usage =
"ComplexPlotLabel is an option that can be used to override the function name \
being used as the plot label in ComplexPlot3D and ComplexContourPlot."

Discontinuities::usage =
"Discontinuities is an option to PlotDiscontinuous that takes a list of points \
at which the function is discontinuous."

DiskSize::usage =
"DiskSize->r is an option to BranchCutPlot."

HorizontalAsymptotes::usage =
"HorizontalAsymptotes is an option for PlotDiscontinuous."

IntegerListPlot::usage =
"IntegerListPlot[list,x1:1,<options>] plots the list of values (assumed to be equally spaced) \
using a histogram-like display.  The x-coordinate of the first point is assumed to be 1 \
unless otherwise specified by the second argument.  If the option Center->False is specified, \
the bins will run from 1-2, 2-3, etc.  If Center->True is specified, they will run from \
0.5-1.5, 1.5-2.5, etc."

NaryPlot::usage =
"NaryPlot[list,b] plots the sequence of nonnegative integers list as a \
set of b-ary bits.  NaryPlot[list,b] also works for a list of nonnegative \
rational numbers, in which case the numeratos are plotted on top and the \
denominators on bottom.  NaryPlot[list,b,c] scales the denominators by a \
factor of c.  The output is typically displayed using \
Show[PixelPerfect[NaryPlot[f/@Range[256]],BlockSize->2]]."

PixelPerfect::usage =
"PixelPerfect[data,<opts>] displays a raster on the screen at a resolution that maps \
individual cells to exactly one pixel.  This also will produce correctly proportioned \
GIFs when exported.  If takes options of Graphics[Raster], e.g., ColorFunction.  \
Additional options include BlockSize->n.  For example, BlockSize->4, \
ColorFunction->(Switch[#,0,Black,1,Red,2,Yellow,3,White]&), \
ColorFunctionScaling->False."

PlotDiscontinuous::usage =
"PlotDiscontinuous[f,{x,x1,x2,...}] plots a graph of the function f[x], \
correctly handling points of discontinuity.  Ranges can also be specified, as in \
PlotDiscontinuous[f,{x,{x1,x2},{x3,x4},...]."

PlotRoots::usage = 
"PlotRoots[f, {x, xmin, xmax}] generates a plot of f \
as a function of x from xmin to xmax and uses the data in the plot to \
isolate any zero crossings.  FindRoot is applied to each zero crossing \
to refine the estimates."

PointStyle::usage =
"PointStyle is an option for PlotRoots."

RadialContourPlot::usage =
"RadialContourPlot[f,{r,rmin,rmax},opts] makes a radial plot \
of f.  Possible options are Contours, PlotPoints, and ColorFunction."

RadialContourPlots::usage =
"RadialContourPlots[f,{r,rmin,rmax},opts] make side-by-side Re, Im, and Abs \
RadialContourPlot's of f."

RasterGraphics::usage =
"RasterGraphics[m] displays the matrix m as a set of rows, one for each generation, \
reversing the usual order of RasterArray.  Cells with 1 are displayed in black; \
cells with 0 are displayed in white.  This routine is a different implementation \
from the Mathematica help browser entry for CellularAutomaton, which ignores \
options and does not allow grid lines."

RealPlot::usage =
"RealPlot[f,{x,xmin,xmax}] is an implementation of Plot that plots the \
function f only at points where all subexpressions of the unevaluated \
input expression are real-valued."

TrigPlot::usage =
"TrigPlot[angle] makes a diagram showing the definition of trigonometric functions \
of the specified angle."

VerticalAsymptotes::usage =
"VerticalAsymptotes is an option for PlotDiscontinuous."


(* Options *)

Options[BranchCutPlot]={
	BranchStyle->{Red,Thickness[.03]},
	DiskSize->0.15
};

Options[ComplexPlot3D]={
	PlotPoints->50,
	Mesh->False,
	ClipFill->None,
	AxesEdge->{{-1,-1},{1,-1},{-1,-1}}, (* needed for Abs[Sign[z]] *)
	ImageSize->500
};

Options[ComplexContourPlot]={
	PlotPoints->200,
	ContourLines->False,
	ColorFunction->Hue,
	ImageSize->500
};

Options[IntegerListPlot]={
	Center->True,
	PlotJoined->True,
	PlotStyle->Red
};

Options[PixelPerfect]={
	PixelsPerCell->Automatic,
	BlockSize->1,
	Blocking->Automatic,
	Frame->0,
	Margin->0,
	Spacing->10,
	MaxSize->800,
	Colors->Automatic,
	ColorFunctionScaling->False,
    MaxPixelsPerCell->1
};

Options[PlotDiscontinuous]={
	PlotStyle->Red,
	Epilog->{},
	HorizontalAsymptotes->{},
	VerticalAsymptotes->{},
	AsymptoteStyle->{Dashing[{.02}]}
};

Options[PlotRoots]={
	PointStyle->{PointSize[.02],Red}
};

Options[RadialContourPlot]={
	Contours->50,
	PlotPoints->15,
	ColorFunction->Hue
(*    ColorFunction->(Hue[Random[]]&)*)
};

Options[RasterGraphics]={
	Mesh->True
};


Begin["`Private`"]

addPointsAndMapToDisk[delta_,Polygon[l_]]:=Module[
	{refinedLine,upperHalfDisk},
    refinedLine=
      Join@@(If[(* add new points if needed *)
      (
        phiDist=Sqrt[#.#&[#[[2]]-#[[1]]]])<delta,#, delta1=#[[2]]-#[[1]];
        phil=Ceiling[phiDist/delta];
        Table[#[[1]]+i/phil delta1,{i,0,phil}]]&/@
        (* line segments *)
        (N[Partition[Append[l,First[l]],2,1]])
      );
    {Polygon[upperHalfDisk=Apply[#2 {Cos[#1],Sin[#1]}&,refinedLine,{1}]],
      Polygon[Map[{1,-1} #&,upperHalfDisk]]}
]

BracketZeros[Line[coords_List]] := 
  Cases[Partition[coords, 2, 1], {{x1_, y1_}, {x2_, y2_}} /; y1 y2 <= 0]

(* Branch Cut Plot *)

OpenCircle[x_,r_:.3]:={{White,Disk[Through[{Re,Im}[x]],r]},Red,
    Circle[Through[{Re,Im}[x]],r]}

ClosedCircle[x_,r_:.3]:={Red,Disk[Through[{Re,Im}[x]],r]}

BranchCutPlot[f_,opts___?OptionQ]:=Module[
    {
      i,
      cuts=BranchCuts[f]/.DirectedInfinity[x_]\[RuleDelayed]999x,
      r=DiskSize/.{opts}/.Options[BranchCutPlot],
      style=BranchStyle/.{opts}/.Options[BranchCutPlot],
      gopts=FilterOptions[Graphics,opts]
      },
    Show[
      Graphics[{
          {
          Sequence@@style,
            cuts/.{i:(_OpenInterval|_Interval|_HalfOpenInterval|_HalfClosedInterval):>
            Line[Through[{Re,Im}[#]]&/@First[i]]}
            },
        cuts/.{OpenInterval[l_]:>(OpenCircle[#,r]&/@l),
              Interval[l_]:>(ClosedCircle[#,r]&/@l),
              HalfOpenInterval[{a_,b_}]:>{OpenCircle[a,r],ClosedCircle[b,r]},
              HalfClosedInterval[{a_,b_}]:>{ClosedCircle[a,r],OpenCircle[b,r]}
              }
          }],gopts,
      Ticks->Table[Range[-3,3],{2}],
      AspectRatio->Automatic,
      PlotRange->{{-1,1},{-1,1}}2.5,
      ImageSize->200,
      Axes->True,
      AxesLabel->(StyleForm[#,FontFamily->"Times",
                FontSlant->"Italic"]&/@{"x","y"})]
    ]

ComplexData[f_,{z_,zmin_,zmax_},npoints_]:=Module[{x,y},
	Map[
		If[MachineNumberQ[#],#,10.^10]&,
		Table[Function[z,f][N[x+I y]],
			{y,Im[zmin],Im[zmax],Im[zmax-zmin]/npoints},
			{x,Re[zmin],Re[zmax],Re[zmax-zmin]/npoints}
		],
		{2}
	]
]

ComplexContourPlot[f_,{z_,zmin_,zmax_},opts___?OptionQ]:=Module[
	{
		x,y,
		copts=FilterOptions[ListContourPlot,opts,Options[ComplexContourPlot]],
		gopts=FilterOptions[GraphicsArray,opts,Options[ComplexContourPlot]],
		label,data,npoints
	},
	label=ComplexPlotLabel/.{opts}/.{ComplexPlotLabel->f};
	npoints=PlotPoints/.{opts}/.Options[ComplexContourPlot];
	(*copts=Sequence@@DeleteCases[{copts},PlotPoints->_];*)
	data=ComplexData[f,{z,zmin,zmax},npoints];
	Show[GraphicsArray[
			ListContourPlot[#[data],
			Evaluate[copts],
			DisplayFunction->Identity,
			MeshRange->Through[{Re,Im}[{zmin,zmax}]],
			FrameLabel->(StyleForm[#,FontFamily->"Times",FontSlant->"Italic"]&/@{"x","y"}),
			PlotLabel->TraditionalForm[#[label]]
		]&/@{Re,Im,Abs},
		Evaluate[gopts],
		GraphicsSpacing->-.05]
	]
]/; Im[zmin]!=0||Im[zmax]!=0

(*
ComplexContourPlot[f_,{z_,zmin_,zmax_},opts___?OptionQ]:=Module[
	{
		x,y,
		copts=FilterOptions[ContourPlot,opts,Options[ComplexContourPlot]],
		gopts=FilterOptions[GraphicsArray,opts,Options[ComplexContourPlot]],
		label
	},
	label=ComplexPlotLabel/.{opts}/.{ComplexPlotLabel->f};
	Show[GraphicsArray[
			ContourPlot[#[Function[z,f][x+I y]],{x,Re[zmin],Re[zmax]},{y,Im[zmin],Im[zmax]},
			Evaluate[copts],
			DisplayFunction->Identity,
			FrameLabel->(StyleForm[#,FontFamily->"Times",FontSlant->"Italic"]&/@{"x","y"}),
			PlotLabel->TraditionalForm[#[label]]
		]&/@{Re,Im,Abs},
		Evaluate[gopts],
		GraphicsSpacing->-.05]
	]
]/; Im[zmin]!=0||Im[zmax]!=0
*)

ComplexContourPlot[f_,{z_,zmin_,zmax_},opts___?OptionQ]:=
	ComplexContourPlot[f,{z,zmin(I+1),zmax(I+1)},opts] /; Im[zmin]==Im[zmax]==0

(*
ComplexPlot3D[f_,{z_,zmin_,zmax_},opts___?OptionQ]:=Module[
	{
		x,y,
		copts=FilterOptions[Plot3D,opts,Options[ComplexPlot3D]],
		gopts=FilterOptions[GraphicsArray,opts,Options[ComplexPlot3D]],
		label
	},
	label=ComplexPlotLabel/.{opts}/.{ComplexPlotLabel->f};
	Show[GraphicsArray[
		Plot3D[#[Function[z,f][x+I y]],{x,Re[zmin],Re[zmax]},{y,Im[zmin],Im[zmax]},
			Evaluate[copts],
			DisplayFunction->Identity,
			AxesLabel->(StyleForm[#,FontFamily->"Times",FontSlant->"Italic"]&/@{"x","y",""}),
			PlotLabel->TraditionalForm[#[label]]
		]&/@{Re,Im,Abs},
		Evaluate[gopts]
		]
	]
]/; Im[zmin]!=0||Im[zmax]!=0
*)

ComplexPlot3D[f_,{z_,zmin_,zmax_},opts___?OptionQ]:=Module[
	{
		x,y,
		copts=FilterOptions[ListPlot3D,opts,Options[ComplexPlot3D]],
		gopts=FilterOptions[GraphicsArray,opts,Options[ComplexPlot3D]],
		label,data,npoints
	},
	label=ComplexPlotLabel/.{opts}/.{ComplexPlotLabel->f};
	npoints=PlotPoints/.{copts}/.Options[ComplexPlot3D];
	(*copts=Sequence@@DeleteCases[{copts},PlotPoints->_];*)
	data=ComplexData[f,{z,zmin,zmax},npoints];
	Show[GraphicsArray[
		ListPlot3D[#[data],
			Evaluate[copts],
			DisplayFunction->Identity,
			MeshRange->Through[{Re,Im}[{zmin,zmax}]],
			AxesLabel->(StyleForm[#,FontFamily->"Times",FontSlant->"Italic"]&/@{"x","y",""}),
			PlotLabel->TraditionalForm[#[label]]
		]&/@{Re,Im,Abs},
		Evaluate[gopts]
		]
	]
]/; Im[zmin]!=0||Im[zmax]!=0

ComplexPlot3D[f_,{z_,zmin_,zmax_},opts___?OptionQ]:=
	ComplexPlot3D[f,{z,zmin(I+1),zmax(I+1)},opts] /; Im[zmin]==Im[zmax]==0

EmptyCircle[x_List?VectorQ, r_:6] := {{AbsolutePointSize[r], Point[x]}, 
      {GrayLevel[1], {AbsolutePointSize[r - 2], Point[x]}}}
      
FilledCircle[x_List?VectorQ, r_:6] := {AbsolutePointSize[r], Point[x]}

IntegerListPlot[l_List?VectorQ,x0_Integer:1,opts___?OptionQ]:=Module[
  {
  ctr=Center/.{opts}/.Options[IntegerListPlot],
  lopts=FilterOptions[ListPlot,opts,Options[IntegerListPlot]],
  off
  },
  off=x0-1+If[ctr,-1/2,0];
  ListPlot[Flatten[
      MapIndexed[{{off+Sequence@@#2,#1[[1]]},{off+Sequence@@#2,#1[[2]]}}&,
        Partition[Prepend[l,0],2,1]],1],Evaluate[lopts]]
]

(* Nary Plot *)

NaryPlot[List[l__Integer],b_Integer:2]:=
  Module[{m=Max[{l}]},
	(*1-*)1/(b-1)Transpose[IntegerDigits[#,b,Ceiling[Log[b,m+1]]]&/@{l}]
]

NaryPlot[l:List[(_Integer|_Rational)..],b_Integer:2,c_:1]:=
	Join[NaryPlot[Numerator[l],b],c Reverse[NaryPlot[Denominator[l],b]]]

(* Pixel Perfect *)

PixelPerfect[datain_,opts___?OptionQ]:=
  Module[{opt,mppc,ppc,bl,fr,ma,sp,size,col,width,height,data,framecol,range},
    data=Switch[Length[Dimensions[datain]],2,{{datain}},3,{datain},4,datain,_,
        Abort[]];
    ropts=FilterOptions[Raster,opts,Options[PixelPerfect]];
    {fr,ma,sp}=Ceiling[{Frame,Margin,Spacing}/.{opts}/.Options[PixelPerfect]];
    blocksize=BlockSize/.{opts}/.Options[PixelPerfect];
    If[IntegerQ[blocksize]&&blocksize>1,data={{ToBlocks[data[[1,1]],blocksize]}}];
    {height,width}=
      With[{dim=Dimensions[data]},{dim[[1]]*dim[[3]],
          dim[[2]]*(dim[[4]]+2*(ma+fr))+(dim[[2]]-1)*sp}];
    {ppc,bl,size,col,mppc}=
      {PixelsPerCell,Blocking,MaxSize,Colors,MaxPixelsPerCell}/.
      {opts}/.Options[PixelPerfect];
    mppc=Ceiling[mppc];
    If[(ppc===Automatic&&bl===Automatic),bl=Ceiling[width/size];
      ppc=Floor[size/(width/bl)],
      If[ppc===Automatic,ppc=Max[1,Floor[size/(width/bl)]],
        If[bl===Automatic,bl=Ceiling[width/(size*ppc)],
          {ppc,bl}=Ceiling[{ppc,bl}]]]];
    If[ppc>mppc,ppc=mppc];
    If[col==Automatic,col=Max[Flatten[data]]+1,col=Ceiling[col]];
    framecol=col-1;
    data=Map[
        Function[y,PadLeft[y,Dimensions[y]+2*fr,framecol,fr]][
            Function[x,PadLeft[x,Dimensions[x]+2*ma,0,ma]][
              If[bl>1,Take[#,{1,-1,bl},{1,-1,bl}],#]]]&,data,{2}];
    data=Map[
        Function[x,
          Flatten/@Map[
              Join[(PadRight[#,Length[#]+sp]&)/@Drop[#,-1],{Last[#]}]&,
              Transpose[x],{1}]],data];
    data=Flatten[
        Join[Function[x,PadRight[x,{Length[x]+sp,Length[x[[1]]]}]]/@
            Drop[data,-1],{Last[data]}],1];
    data=Reverse[(*col-1-*)data];
    range=Subtract@@Through[{Max,Min}[Flatten[data]]];
    Graphics[Raster[data (*data/(col-1)*),
 	 	  ropts,
    	   ColorFunction->(GrayLevel[1-#/range]&)
		],
        PlotRange->({0,#}&/@Reverse[Dimensions[data]]),
        AspectRatio->Automatic,
        ImageSize->If[bl==ppc==1&&ppc*width>size,
          size,
          Reverse[Dimensions[data]]*ppc+1]
    ]
]

OpenPoint[x_,r_:7]:={ClosedPoint[x,r],{White,AbsolutePointSize[r-2],Point[x]}}
ClosedPoint[x_,r_:7]:={Red,AbsolutePointSize[r],Point[x]}

PlotDiscontinuous[f_,{x_Symbol,ranges_List?MatrixQ},opts___?OptionQ]:=Module[
	{
		horiz=HorizontalAsymptotes/.{opts}/.Options[PlotDiscontinuous],
		vert=VerticalAsymptotes/.{opts}/.Options[PlotDiscontinuous],
		popts=FilterOptions[Plot,opts,Options[PlotDiscontinuous]],
		gopts=FilterOptions[Graphics,opts],
		epi,epi2,
		astyle=AsymptoteStyle/.{opts}/.Options[PlotDiscontinuous],
		eps=10.^-15,
		pts,pts2
	},
	epi={
			{
			Sequence@@Flatten[{astyle}],
			Function[Line[{{#,-99},{#,99}}]]/@vert,
			Function[Line[{{-99,#},{99,#}}]]/@horiz
			},
		Epilog/.{opts}/.Options[PlotDiscontinuous]
	};
	pts={#[[1,1]],Last/@#}&/@Split[Union[Join[
          Flatten[{{#,0},{#,-1}}&/@First/@ranges,1],
          Flatten[{{#,0},{#,1}}&/@Last/@ranges,1]
          ]],First[#1]==First[#2]&];
	pts2=DeleteCases[
		Append[#,Function[x,f]/@(First[#]-eps Last[#])]&/@pts,
		_?(!FreeQ[#,ComplexInfinity]&)];
	(*Print[pts2//N//InputForm];*)
	epi2=Switch[#2,
		{-1,0,1}, (* both boundaries *)
			{
			If[Unequal@@N[#3],
				{OpenPoint[{#1,#3[[1]]}],OpenPoint[{#1,#3[[-1]]}],ClosedPoint[{#1,#3[[2]]}]},{}],
			If[#3[[1]]==#3[[2]]!=#3[[3]],
				{ClosedPoint[{#1,#3[[1]]}],OpenPoint[{#1,#3[[3]]}]},{}],
			If[#3[[1]]!=#3[[2]]==#3[[3]],
				{OpenPoint[{#1,#3[[1]]}],ClosedPoint[{#1,#3[[2]]}]},{}]
			},
		{-1,0}, (* left boundary *)
			If[Unequal@@N[#3],
				OpenPoint[{#1,#3[[1]]}],
			(* Else *)
				If[#1!=pts[[1,1]]||(#1==pts[[1,1]]&&Function[x,f][#1-eps]!=#3[[1]]),
					ClosedPoint[{#1,#3[[1]]}],{}]
			],
		{0,1}, (* right boundary *)
			If[Unequal@@N[#3],
				OpenPoint[{#1,#3[[2]]}],
			(* Else *)
				If[#1!=pts[[-1,1]]||(#1==pts[[-1,1]]&&Function[x,f][#1+eps]!=#3[[1]]),
					ClosedPoint[{#1,#3[[2]]}],{}]
			]
	]&@@@pts2;
	Show[
		Plot[f,Evaluate[{x,Sequence@@#}],
			Evaluate[popts],
			DisplayFunction->Identity
		]&/@ranges,
		Evaluate[Epilog->{epi,epi2}],
		Evaluate[gopts],
		DisplayFunction->$DisplayFunction
	]
] /; Dimensions[ranges][[-1]]==2

PlotDiscontinuous[f_,{x_Symbol,endpoints__?(NumericQ[#]&&Im[#]==0&)},opts___?OptionQ]:=
	PlotDiscontinuous[f,{x,Partition[{endpoints},2,1]},opts] /;
		Less[endpoints]

PlotDiscontinuous[f_,{x_Symbol,intervals__?VectorQ},opts___?OptionQ]:=
	PlotDiscontinuous[f,{x,Flatten[Partition[#,2,1]&/@{intervals},1]},opts] /;
		And@@((NumericQ[#]&&Im[#]==0)&/@Flatten[{intervals}]) &&
			LessEqual@@Flatten[{intervals}]

PlotRoots[f_, {x_, begin_, end_}, opts___?OptionQ] := 
  Module[{plot, lines, bracket, froot, roots,
  	pointstyle={PointStyle/.Options[PlotRoots],PointStyle/.{opts}/.PointStyle->{}}
  },
  	Block[{$DisplayFunction=Identity},
    plot = Plot[f, {x, begin, end},
    	Evaluate[FilterOptions[Plot, opts]]]
    ];
    lines = Cases[plot, _Line, Infinity];
    brackets = Flatten[Map[BracketZeros, lines], 1];
    froot[{{x1_, y1_}, {x2_, y2_}}] := 
      FindRoot[f, {x, x1, x2}, Evaluate[FilterOptions[FindRoot, opts]]];
    roots=x/.Map[froot, brackets];
    Show[plot,Epilog->{Sequence@@Flatten[{pointstyle}],
    	Point/@(Function[x,{x,Evaluate[f]}]/@roots)}];
    roots
]

(* Radial Contour Plot *)

RadialContourPlot[f_,{r_,rmin_,rmax_},opts___?OptionQ]:=With[{eps=10.^-10},
	Module[
	{
		data,t,ppr,ppt,n1,n2,cls,gr,
        contours,pp,colorfunction,
        gropts=FilterOptions[Graphics,opts]
	},
	{contours,pp,colorfunction}={Contours,PlotPoints,ColorFunction}/.
		{opts}/.Options[RadialContourPlot];
	{ppr,ppt}=If[NumberQ[pp],{pp,pp},pp];
	n1=Max[Round[ppr ppt/contours],1];
	n2=Max[Round[n1/2],1];
		data=Table[Function[r,f][r Exp[I t]],
			{r,rmin,rmax,(rmax-rmin)/ppr},{t,eps,Pi-eps,(2Pi-2eps)/ppt}];
	cls=#[[n2]]&/@Partition[Sort[Flatten[data]],n1];
	gr=Graphics[ListContourPlot[data,
		MeshRange->{N[{eps,Pi-eps}],{rmin,rmax}},Contours->cls,
		ColorFunction->(colorfunction),ContourLines->False,
		DisplayFunction->Identity]];
	Show[Graphics[{{(colorfunction)[Function[r,f][#[[1,1,1]]+#[[1,1,2]]I]],
			addPointsAndMapToDisk[0.1,#]}&/@
		Cases[gr,_Polygon,Infinity],{GrayLevel[0],Thickness[0.01],
		Circle[{0,0},rmax]}}],
		Evaluate[gropts],
		DisplayFunction->$DisplayFunction,AspectRatio->Automatic]
	]
]
    
RadialContourPlots[f_,{r_,rmin_,rmax_},opts___?OptionQ]:=Module[
	{
		label=f
	},
	Show[GraphicsArray[RadialContourPlot[#[f],{r,rmin,rmax},opts,
		DisplayFunction->Identity,
		FrameLabel->(StyleForm[#,FontFamily->"Times",FontSlant->"Italic"]&/@{"x","y"}),
		PlotLabel->TraditionalForm[#[label]]
			]&/@{Re,Im,Abs}]]
]

(*
RasterGraphics[(state_)?MatrixQ, colors_Integer:2, opts___?OptionQ] := 
  Graphics[Raster[Reverse[1 - state/(colors - 1)]], Frame -> True, 
   FrameTicks -> None, AspectRatio -> (AspectRatio /. {opts} /. 
     AspectRatio -> Automatic), opts]
*)

RasterGraphics[s_List?MatrixQ,colors_Integer:2,opts___?OptionQ]:=DensityGraphics[
  Reverse[1 - s/(colors - 1)],
  opts,
  AspectRatio->Automatic,
  Frame->False,
  Mesh->True,
  ColorFunctionScaling->False
]

(* RealPlot *)

SetAttributes[NumericSubexpr,HoldAll];
NumericSubexpr[e_,bag_]:=(Internal`StuffBag[bag,e];
    If[Hold[e][[1,0,0]]===Symbol&&
        MemberQ[Attributes[Evaluate[Hold[e][[1,0]]]],NumericFunction],
      Scan[NumericSubexpr[#,bag]&,Hold[e],{2}]];)

SetAttributes[RealPlot,HoldAll];
RealPlot[f_,args___?OptionQ]:=Module[{bag},bag=Internal`Bag[];
    NumericSubexpr[f,bag];
    bag=Internal`BagPart[bag,All];
    Internal`DeactivateMessages[
      Plot[If[Element[bag,Reals],f,Indeterminate],args],Plot::plnr]]

(* ToBlocks *)

ToBlocks[m_?MatrixQ,n_]:=BlockMatrix[Map[Table[#,{n},{n}]&,m,{2}]]

(* TrigPlot *)

TrigPlot[th_,opts___?OptionQ]:=Module[
    {
      p=Through[{Cos,Sin}[th]],
      pos=AnglePosition/.{opts}/.(AnglePosition->.35Through[{Cos,Sin}[th/2]]),
      eps=.02
    },
    Show[Graphics[
        {
          Circle[{0,0},1,{th,2\[Pi]}],
          {Red,Line[{p,{0,0},{1,0}}],
            {Dashing[{.02}],Line[{{Cos[th],0},p}]},
            PointSize[.03],Point[{0,0}]
            },
          {Blue,Circle[{0,0},1,{0,th}],PointSize[.03],Point[p]},
          Text[HoldForm["\[Theta]"==th]//TraditionalForm,pos,Background->White],
          Text[Cos["\[Theta]"]//TraditionalForm,{Cos[th]/2,-eps},{0,1},Background->White],
          Text[Sin["\[Theta]"]//TraditionalForm,{eps+Cos[th],Sin[th]/2},{-1,0},Background->White]
          }
        ],AspectRatio\[Rule]Automatic,Axes\[Rule]True,Ticks->None,
      ImageSize\[Rule]200,PlotRange\[Rule]All]
    ]

End[]

(* Protect[  ] *)

EndPackage[]